@@ -57,6 +57,8 @@
 #include "ngtcp2_ppe_test.h"
 #include "ngtcp2_dcidtr_test.h"
 #include "ngtcp2_addr_test.h"
+#include "ngtcp2_pcg_test.h"
+#include "ngtcp2_ratelim_test.h"
 
 int main(int argc, char *argv[]) {
   const MunitSuite suites[] = {
@@ -73,7 +75,8 @@ int main(int argc, char *argv[]) {
     qlog_suite,     window_filter_suite,
     settings_suite, callbacks_suite,
     ppe_suite,      dcidtr_suite,
-    addr_suite,     {0},
+    addr_suite,     pcg_suite,
+    ratelim_suite,  {0},
   };
   const MunitSuite suite = {
     .prefix = "",
